@@ -20,6 +20,8 @@
  *
  */
 
+#include "config.h"
+
 #include <malloc.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -32,7 +34,6 @@
 #include <errno.h>
 
 #include "cdrwtool.h"
-#include "config.h"
 #include "options.h"
 
 #include "../mkudffs/defaults.h"
@@ -40,10 +41,12 @@
 
 int write_func(struct udf_disc *disc, struct udf_extent *ext)
 {
-	static char *buffer = NULL;
-	static int bufferlen = 0, lastpacket = -1;
+	static unsigned char *buffer = NULL;
+	static size_t bufferlen = 0;
+	static int lastpacket = -1;
 	int fd = *(int *)disc->write_data;
-	int offset, packet;
+	size_t offset;
+	int packet;
 	struct udf_desc *desc;
 	struct udf_data *data;
 
