@@ -8,24 +8,24 @@
  *	When a new buffer is required preferentially an unused not-dirty buffer gets overwritten.
  *	If not available, then a unused dirty buffer gets written and is then reused.
  *	If no such buffer can be found the system panics.
- *  
- * CONTACTS
- *	E-mail regarding this program may be directed to 
- *		e.fennema@dataweb.nl
  *
  * COPYRIGHT
  *	This file is distributed under the terms of the GNU General Public
  *	License (GPL). Copies of the GPL can be obtained from:
  *		ftp://prep.ai.mit.edu/pub/gnu/GPL
  *
- *  (C) 2001 Enno Fennema
+ *  (C) 2001 Enno Fennema <e.fennema@dataweb.nl>
  *
  * HISTORY
  *  16 Aug 01  ef  Created.
  */
 
+#include "config.h"
+
 #include <stdlib.h>
 #include <stdio.h>
+#include <errno.h>
+#include <string.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <sys/stat.h>
@@ -43,7 +43,7 @@ struct packetbuf {
     uint32_t		dirty;
     uint32_t		bufNum;
     uint32_t		start;
-    uint8_t		*pkt;
+    unsigned char	*pkt;
 };
 
 int			lastTrack;
@@ -55,8 +55,8 @@ u_char *cp_buffer;
 
 struct packetbuf pktbuf[MAXPKTBUFS+1];			/* last one special use */
 
-uint8_t	*verifyBuffer;					/* for verify only */
-uint8_t	*blockBuffer;
+unsigned char	*verifyBuffer;					/* for verify only */
+unsigned char	*blockBuffer;
 
 uint32_t	trackStart;
 uint32_t	trackSize;
@@ -89,7 +89,7 @@ void markBlock(enum markAction action, uint32_t blkno) {
 }
 
 /*	GetExtents()
- *	Try to find unallocated blocks for 'requestedLength' bytes in the rewriteable partition
+ *	Try to find unallocated blocks for 'requestedLength' bytes in the rewritable partition
  *	Return short_ad's of lbns in the physical partition together satisfying that request
  *	Last extent has length < n * 2048; if exact multiple of 2048 the a final length 0 short_ad
  *
@@ -195,8 +195,8 @@ int freeLongExtents(long_ad* extents) {
  *	Only used to get new extent for LVID sequence
  *	(Could use to extend sparing blocks as well)
  */
-void getUnallocSpaceExtent(int requestLength, int requestAfter, extent_ad *alloc) {
-    int		i, n;
+void getUnallocSpaceExtent(uint32_t requestLength, uint32_t requestAfter, extent_ad *alloc) {
+    uint32_t i, n;
 
     alloc->extLength = alloc->extLocation = 0;
     n = (requestLength + 2047) & ~2047;
@@ -326,7 +326,7 @@ uint32_t lookupSparingTable(uint32_t original) {
 uint32_t newSparingTableEntry(uint32_t original) {
     struct sparingEntry *se, newEntry, swapEntry;
     uint32_t	mapped;
-    int		i;
+    unsigned int	i;
 
     if( !st ) {
 	printf("No sparing table provided\n");
@@ -369,12 +369,15 @@ uint32_t newSparingTableEntry(uint32_t original) {
  *	Do not verify writing as that would change the table again.
  */
 void updateSparingTable() {
-    int			i, pbn, stat;
+    size_t		i;
+    int			pbn, ret;
+    off_t		off;
+    ssize_t		len;
     struct generic_desc	*p;
     struct packetbuf	*pb;
     struct sparablePartitionMap *spm = (struct sparablePartitionMap*)lvd->partitionMaps;
 
-    for( i = 0; i < sizeof(spm->locSparingTable); i++ ) {
+    for( i = 0; i < sizeof(spm->locSparingTable)/sizeof(spm->locSparingTable[0]); i++ ) {
 	pbn = spm->locSparingTable[i];
 	if( pbn == 0 )
 	    return;
@@ -387,14 +390,18 @@ void updateSparingTable() {
 	    sizeof(struct sparingTable) + st->reallocationTableLen * sizeof(struct sparingEntry) - sizeof(tag);
 	setChecksum(p);
 	if( devicetype != DISK_IMAGE ) {
-	    stat = writeCD(device, pbn, 32, pb->pkt);
-	    if( stat )
+	    ret = writeCD(device, pbn, 32, pb->pkt);
+	    if( ret )
 		printf("Write SparingTable at %d: %s\n", pbn, get_sense_string());
 	} else { // DISK_IMAGE
-	    stat= lseek(device, 2048 * pbn, SEEK_SET);
-	    stat = write(device, pb->pkt, 32 * 2048);
-	    if( stat < 0 )
+	    off = lseek(device, 2048 * pbn, SEEK_SET);
+	    if( off == (off_t)-1 )
 		fail("writeSparingTable at %d: %m\n", pbn);
+	    len = write(device, pb->pkt, 32 * 2048);
+	    if( len < 0 )
+		fail("writeSparingTable at %d: %m\n", pbn);
+	    if( len != 32 * 2048 )
+		fail("writeSparingTable at %d: %s\n", pbn, strerror(EIO));
 	}
     }
 }
@@ -403,29 +410,38 @@ void updateSparingTable() {
 int 
 readPacket(struct packetbuf* pb) 
 {
-    int		stat;
+    int		ret;
+    off_t	off;
+    ssize_t	len;
     uint32_t	physical;
 
     physical = lookupSparingTable(pb->start);
 
     if( devicetype != DISK_IMAGE ) {
-	stat = readCD(device, sectortype, physical, 32, pb->pkt);
-	if( stat )
+	ret = readCD(device, sectortype, physical, 32, pb->pkt);
+	if( ret )
 	    printf("readPacket: readCD %s\n", get_sense_string());
     } else {
-	stat = lseek(device, 2048 * physical, SEEK_SET);
-	stat = read(device, pb->pkt, 32 * 2048);
-	if( stat != 32 * 2048 )
+	off = lseek(device, 2048 * physical, SEEK_SET);
+	if( off == (off_t)-1 )
+	    fail("readPacket: lseek failed %m\n");
+	len = read(device, pb->pkt, 32 * 2048);
+	if( len < 0 )
 	    fail("readPacket: read failed %m\n");
+	if( len != 32 * 2048 )
+	    fail("readPacket: read failed %s\n", strerror(EIO));
+	ret = 0;
     }
-    return stat;
+    return ret;
 }
    
 
 int 
 writePacket(struct packetbuf* pb) 
 {
-    int		stat, retry;
+    int		ret, retry;
+    off_t	off;
+    ssize_t	len;
     uint32_t	physical;
 
     pb->dirty = 0;
@@ -436,18 +452,18 @@ writePacket(struct packetbuf* pb)
 	    if( retry != 0 )
 		physical = newSparingTableEntry(pb->start);
 
-	    stat = writeCD(device, physical, 32, pb->pkt);
+	    ret = writeCD(device, physical, 32, pb->pkt);
 
-	    if( stat )
+	    if( ret )
 		fail("writePacket: writeCD %s\n", get_sense_string());
 
 	    // must now verify write operation and spare if necessary
 	    // My HP8100 does not support Verify or WriteAndVerify
 	    // Use ReadCD with strict Read Error Recovery Parameters
 	    setStrictRead(1);
-	    stat = readCD(device, sectortype, physical, 32, verifyBuffer);
+	    ret = readCD(device, sectortype, physical, 32, verifyBuffer);
 
-	    if( stat == 0 ) {
+	    if( ret == 0 ) {
 		setStrictRead(0);
 		return 0;
 	    }
@@ -461,12 +477,17 @@ writePacket(struct packetbuf* pb)
 		physical = newSparingTableEntry(pb->start);
 	}
 #endif
-	stat= lseek(device, 2048 * physical, SEEK_SET);
-	stat = write(device, pb->pkt, 32 * 2048);
-	if( stat < 0 )
+	off = lseek(device, 2048 * physical, SEEK_SET);
+	if( off == (off_t)-1 )
 	    fail("writePacket: writeHD failed %m\n");
+	len = write(device, pb->pkt, 32 * 2048);
+	if( len < 0 )
+	    fail("writePacket: writeHD failed %m\n");
+	if( len != 32 * 2048 )
+	    fail("writePacket: writeHD failed %s\n", strerror(EIO));
+	ret = 0;
     }
-    return stat;
+    return ret;
 }
 
 
@@ -542,20 +563,24 @@ readBlock(uint32_t lbn, uint16_t part)
 void* 
 readSingleBlock(uint32_t pbn) 
 {
-    int stat;
+    int ret;
+    off_t off;
+    ssize_t len;
 
     if( devicetype != DISK_IMAGE ) {
-	stat = readCD(device, sectortype, pbn, 1, blockBuffer);
-	if( stat ) {
+	ret = readCD(device, sectortype, pbn, 1, blockBuffer);
+	if( ret ) {
 	    if( ! ignoreReadError )
 		printf("readSingleBlock: %s\n", get_sense_string());
     	    return NULL;
 	} else 
 	    return blockBuffer;
     } else {
-	stat = lseek(device, 2048 * pbn, SEEK_SET);
-	stat = read(device, blockBuffer, 2048);
-	if( stat != 2048 )
+	off = lseek(device, 2048 * pbn, SEEK_SET);
+	if( off != (off_t)-1 )
+	    return NULL;
+	len = read(device, blockBuffer, 2048);
+	if( len != 2048 )
 	    return NULL;
 	else
 	    return blockBuffer;
@@ -630,7 +655,7 @@ readTaggedBlock(uint32_t lbn, uint16_t partition)
 
     if( block->descTag.tagIdent == 0x0000 ) {
 	/* ignore not ISO 13346 defined descriptors with 0 tags */
-	if( strncmp(((struct sparingTable*)block)->sparingIdent.ident, UDF_ID_SPARING, strlen(UDF_ID_SPARING)) != 0 ) {
+	if( strncmp((char *)((struct sparingTable*)block)->sparingIdent.ident, UDF_ID_SPARING, strlen(UDF_ID_SPARING)) != 0 ) {
 	    for( i = 0; i < 2048; i++ ) {
 		if( ((uint8_t*)block)[i] != 0 ) {
 		    printf("readTaggedBlock: Empty block %d not all zeroes\n", lbn);
@@ -755,6 +780,8 @@ initIO(char *filename)
 {
     struct packetbuf *pb;
     int		rv;
+    off_t	off;
+    ssize_t	len;
     struct stat filestat;
     struct cdrom_discinfo  di;
     u_char *buffer;
@@ -766,15 +793,31 @@ initIO(char *filename)
 
     if( S_ISREG(filestat.st_mode) ) {		/* disk image of a UDF volume */
 	devicetype = DISK_IMAGE;
-	if( !(device = open(filename, O_RDWR)) ) {
+	if( (device = open(filename, O_RDWR)) < 0 ) {
 	    fail("initIO: open %s failed\n", filename);
 	    return 0;
 	}
 
 	/* heuristically determine medium imitated on disk image based on VAT FileEntry in block 512 */
-	rv = lseek(device, 2048 * 512, SEEK_SET);
-	rv = read(device, &ident, 2);
+	off = lseek(device, 2048 * 512, SEEK_SET);
+	if( off == (off_t)-1 )
+	    fail("initIO: lseek %s failed: %m\n", filename);
+	len = read(device, &ident, 2);
+	if( len < 0 )
+	    fail("initIO: read %s failed: %m\n", filename);
+	if( len != 2 )
+	    fail("initIO: read %s failed: %s\n", filename, strerror(EIO));
 	medium = ident == TAG_IDENT_VDP ? CDR : CDRW;
+
+	if( medium == CDRW ) {
+	    for( pb = pktbuf, rv = 1; pb <= pktbuf + MAXPKTBUFS; pb++ ) {
+		pb->start = 0xFFFFFFFF;
+		pb->pkt = malloc(32*2048);
+		pb->bufNum = rv++;
+		if( pb->pkt == NULL )
+		    fail("malloc packetBuffer failed\n");
+	    }
+	}
     }
 
     if( (blockBuffer = malloc(2048)) == NULL )
@@ -783,7 +826,7 @@ initIO(char *filename)
     if( devicetype == DISK_IMAGE )
 	return 0;
 
-    if( !(device = open(filename, O_RDONLY | O_NONBLOCK )) )
+    if( (device = open(filename, O_RDONLY | O_NONBLOCK )) < 0 )
 	fail("initIO: open %s failed\n", filename);
 
     rv = ioctl(device, CDROM_DRIVE_STATUS);
@@ -804,7 +847,7 @@ initIO(char *filename)
 
     lastTrack = di.trk1_lastsession;
 
-    if( read_trackinfo(device, &ti, lastTrack) )	/* last track 1 in last session writeable for UDF */
+    if( read_trackinfo(device, &ti, lastTrack) )	/* last track 1 in last session writable for UDF */
 	fail("Read discinfo failed\n");
 
     if( medium == CDRW ) {
@@ -832,7 +875,7 @@ initIO(char *filename)
 	    fail("CDR disc last track reserved or not variable packet\n");
 
 	if( !ti.nwa_v )
-	    fail("Next writeable address invalid in track %d\n", ti.trk);
+	    fail("Next writable address invalid in track %d\n", ti.trk);
     }
 
     trackStart = ti.trk_start;
